// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "assert/assert.hpp"
#include "UIRowContainer.hpp"

using namespace Dawn;

std::vector<std::shared_ptr<UIElement>> UIRowContainer::getChildren() {
  return this->children;
}

void UIRowContainer::appendChild(std::shared_ptr<UIElement> child) {
  assertNotNull(child, "Cannot append a null child!");
  this->children.push_back(child);
}

void UIRowContainer::removeChild(std::shared_ptr<UIElement> child) {
  assertNotNull(child, "Cannot remove a null child!");
  auto it = std::find(this->children.begin(), this->children.end(), child);
  if(it == this->children.end()) return;
  this->children.erase(it);
}

void UIRowContainer::clearChildren() {
  this->children.clear();
}

void UIRowContainer::updateAlignment(
  const glm::vec2 parentPosition,
  const glm::vec2 parentSize,
  const float_t canvasScale
) {
  this->updateSelfAlignment(parentPosition, parentSize, canvasScale);

  // Now we have our dimensions, divide evenly
  auto children = this->getChildren();

  float_t y = 0.0f;
  float_t yPiece = this->size.y / (float_t)children.size();
  
  // Update all children
  for(auto &child : this->children) {
    child->updateAlignment(
      this->position + glm::vec2(0, y),
      glm::vec2(
        this->size.x,
        yPiece
      ),
      canvasScale
    );
    y += yPiece;
  }
}