
// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "scene/SceneList.hpp"
#include "component/display/Camera.hpp"
#include "prefab/SimpleSpinningCube.hpp"
#include "display/font/TrueTypeTexture.hpp"
#include "component/display/material/SimpleTexturedMaterial.hpp"
#include "display/mesh/QuadMesh.hpp"

#include "component/ui/UICanvas.hpp"
#include "ui/UIRectangle.hpp"
#include "ui/UILabel.hpp"
#include "ui/UIMenu.hpp"
#include "ui/container/UIRowContainer.hpp"
#include "ui/container/UIGroupContainer.hpp"

#include <ft2build.h>
#include FT_FREETYPE_H

using namespace Dawn;

std::shared_ptr<TrueTypeTexture> texture;

void Dawn::helloWorldScene(Scene &s) {
  std::string font = "ysabeau_medium";
  texture = s.getGame()->assetManager.get<TrueTypeTexture>(font, 24);

  while(!s.getGame()->assetManager.isLoaded(font)) {
    s.getGame()->assetManager.update();
  }

  auto cameraItem = s.createSceneItem();
  auto camera = cameraItem->addComponent<Camera>();
  cameraItem->lookAt({ 120, 0, 300 }, { 120, 0, 0 }, { 0, 1, 0 });
  camera->clipFar = 99999.99f;

  // auto quad = s.createSceneItem();
  // auto quadMesh = std::make_shared<Mesh>();

  // glm::vec2 position = { 0, 0 };
  // glm::vec2 size = texture->bufferStringToMesh(
  //   quadMesh,
  //   L"Hello World!",
  //   position,
  //   true  
  // );

  // auto quadRenderer = quad->addComponent<MeshRenderer>();
  // quadRenderer->mesh = quadMesh;

  // auto quadMaterial = quad->addComponent<SimpleTexturedMaterial>();
  // quadMaterial->setTexture(texture->texture);

  auto uiCanvasItem = s.createSceneItem();
  auto uiCanvas = uiCanvasItem->addComponent<UICanvas>();

  auto container = std::make_shared<UIGroupContainer>();
  container->align = { 32, 32, 300, 250 };
  container->alignX = UIAlignmentType::START;
  container->alignY = UIAlignmentType::START;
  uiCanvas->addElement(container);

  auto rect = std::make_shared<UIRectangle>();
  rect->color = COLOR_MAGENTA;
  container->appendChild(rect);

  auto menu = std::make_shared<UIMenu>();
  menu->setSize(1, 4);
  container->appendChild(menu);

  auto rowContainer = std::make_shared<UIRowContainer>();
  menu->children.push_back(rowContainer);

  auto label0 = std::make_shared<UILabel>();
  label0->setText(L"New Game");
  label0->setFont(texture);
  rowContainer->appendChild(label0);

  auto label1 = std::make_shared<UILabel>();
  label1->setText(L"Load Game");
  label1->setFont(texture);
  rowContainer->appendChild(label1);

  auto label2 = std::make_shared<UILabel>();
  label2->setText(L"Options");
  label2->setFont(texture);
  rowContainer->appendChild(label2);

  auto label3 = std::make_shared<UILabel>();
  label3->setText(L"Exit");
  label3->setFont(texture);
  rowContainer->appendChild(label3);
}